local fs = require "nixio.fs"
local targets = {['name'] = 'modbus-mqtt'}

m = Map("modbus-to-mqtt", translate("Modbus to MQTT"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable Modbus to MQTT"))
option_enabled.rmempty = false
option_enabled.default = "1"

s = m:section(TypedSection, "modbus_mqtt", translate("Modbus MQTT"), "")
s.anonymous = true
s.addremove = false

s:tab("general", translate("General Configuration"))
s:tab("advanced", translate("Advanced Configuration"))

json_format = s:taboption("general",ListValue, "json_format", "Payload JSON Format", translate("Include add modbus data or Only Parameter Values in Published payload"))
json_format:value("full", translate("Full Data"))
json_format:value("lite", translate("Only Parameter Data"))
json_format.default = "full"

conection_type = s:taboption("general",ListValue, "conection_type", translate("Connection Type"))
conection_type:value("non-tls", translate("Non TLS"))
conection_type:value("tls", translate("TLS"))
conection_type.default = "non-tls"

ca_file_name = s:taboption("advanced",Value, "ca_file_name", translate("CA Certificate File Name"), translate(""))
ca_file_name.datatype = "string"
ca_file_name:depends("conection_type", "tls")

ca_file = s:taboption("advanced",FileUpload, "ca_file", translate("Upload CA Certificate"), translate(""))
ca_file.enable_remove = false
ca_file.root_directory = "/etc/luci-uploads/"
ca_file.readonly = true
ca_file:depends("conection_type", "tls")

client_file_name = s:taboption("advanced",Value, "client_file_name", translate("Client Certificate File Name"), translate(""))
client_file_name.datatype = "string"
client_file_name:depends("conection_type", "tls")

client_cert = s:taboption("advanced",FileUpload, "client_cert", translate("Upload Client Certificate"), translate(""))
client_cert.enable_remove = false
client_cert.root_directory = "/etc/luci-uploads/"
client_cert.readonly = true
client_cert:depends("conection_type", "tls")

client_key_name = s:taboption("advanced",Value, "client_key_name", translate("Client Key File Name"), translate(""))
client_key_name.datatype = "string"
client_key_name:depends("conection_type", "tls")

client_key = s:taboption("advanced",FileUpload, "client_key", translate("Upload Client Key File"), translate(""))
client_key.enable_remove = false
client_key.root_directory = "/etc/luci-uploads/"
client_key.readonly = true
client_key:depends("conection_type", "tls")

verify_server = s:taboption("general",ListValue, "verify_server", translate("Verify Server"),translate("Verify Server Certificate"))
verify_server:value("1", translate("Yes"))
verify_server:value("0", translate("No"))
verify_server.default = "1"
verify_server:depends("conection_type", "tls")


host = s:taboption("general",Value, "host", "Host", translate("MQTT broker hostname or IP"))
host.datatype = "host"
host.default = "127.0.0.1"

port = s:taboption("general",Value, "port", "Port", translate("MQTT broker port number"))
port.datatype = "port"
port.default = "1883"
port:depends("conection_type", "non-tls")

port_tls = s:taboption("general",Value, "port_tls", "Port", translate("MQTT broker port number"))
port_tls.datatype = "port"
port_tls.default = "8883"
port_tls:depends("conection_type", "tls")

user = s:taboption("general",Value, "user", "Username", translate("MQTT client username (leave empty if anonymous)"))
user.datatype = "uciname"

pass = s:taboption("general",Value, "pass", "Password", translate("MQTT client password (leave empty if anonymous)"))
pass.password = true
--pass.datatype = "regex('^[a-zA-Z0-9!@#$%%&*+/=?^_`{|}~.-]+$',5)"
function pass:parse(section, novld, ...)
    local u = user:formvalue(section)
    local p = pass:formvalue(section)
    local uEmpty = (u == nil or u == "")
    local pEmpty = (p == nil or p == "")
    if (uEmpty and not pEmpty) or (not uEmpty and pEmpty) then
        self:add_error(section, "invalid", "Both username and password must be either empty or filled")
    end
    Value.parse(self, section, novld, ...)
end

data_topic = s:taboption("general",Value, "data_topic", translate("Data Topic"), "Topic to send Modbus Data")
data_topic.datatype = "string"

client_id = s:taboption("general",Value, "client_id", "Client Id", translate("Leave blank to generate random client id on connection"))
client_id.datatype = "string"

qos = s:taboption("general",Value, "qos", "QoS (Quality Of Service)")
qos.datatype = "range(0,2)"
qos.default = "1"

clean_session = s:taboption("advanced",ListValue, "clean_session", "Clean Session", translate("Tells MQTT Broker if it should discard existing session"))
clean_session:value("1", translate("Yes"))
clean_session:value("0", translate("No"))

retain = s:taboption("advanced",ListValue, "retain", "Retain", translate("Makes the message retained."))
retain:value("1", translate("Yes"))
retain:value("0", translate("No"))
retain.default = "0"

command_topic = s:taboption("advanced",Value, "command_topic", "Command Topic", translate("Topic on which Modbus commands are received"))
command_topic.datatype = "string"

command_response_topic = s:taboption("advanced",Value, "command_response_topic", "Command Response Topic", translate("Topic on which Modbus command responses are sent"))
command_response_topic.datatype = "string"

will_topic = s:taboption("advanced",Value, "will_topic", "Will Topic")
will_topic.datatype = "string"

will_payload = s:taboption("advanced",Value, "will_payload", "Will Payload", translate("Payload to publish in case connection is interrupted"))
will_payload.datatype = "string"

will_qos = s:taboption("advanced",Value, "will_qos", "Will QoS")
will_qos.datatype = "range(0,2)"

will_retain = s:taboption("advanced",ListValue, "will_retain", "Will Retain", translate("Should will message be retained"))
will_retain:value("1", translate("Yes"))
will_retain:value("0", translate("No"))

reconnect_delay = s:taboption("advanced",Value, "reconnect_delay", "Reconnect Delay", translate("Number of seconds to wait before retrying to connect broken connection"))
reconnect_delay.datatype = "range(1,900)"
reconnect_delay.default = 5

reconnect_delay_max = s:taboption("advanced",Value, "reconnect_delay_max", "Reconnect Delay Max", translate("Maximum number of seconds to wait before retrying"))
reconnect_delay_max.datatype = "range(1,3600)"
reconnect_delay_max.default = 900

reconnect_exponential_backoff = s:taboption("advanced",ListValue, "reconnect_exponential_backoff", "Use Exponential Backoff", translate("Should exponential backoff strategy is used when retrying"))
reconnect_exponential_backoff:value("1", translate("Yes"))
reconnect_exponential_backoff:value("0", translate("No"))
reconnect_exponential_backoff.default = "0"

max_inflight_messages = s:taboption("advanced",Value, "max_inflight_messages", "Maximum Inflight Messages", translate("Maximum inflight messages"))
max_inflight_messages.datatype = "range(1,100)"
max_inflight_messages.default = 20

return m
